
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% FILE: mems_thermoelast.m
% Linear elastic analyses for plane problems with linear and quadratic triangles
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
function mems

addpath(genpath('.'))

clear all
global displ analysis coor connec stress     % global variables for post-processing                                       

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% preprocessor phase: reads input from file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%[fname,pname]=uigetfile('input/*.inp',...    % gets name of input file
%                        'Choose input file');

fname='spring.inp';     % fixed input file (mesh can be modified)
%fname='springwith.inp';        % fixed input file (mesh can be modified)
%fname='mems.inp';              % fixed input file (mesh can be modified)
pname='input/';
                                                                                    
%-----------------------------------------------------------------------------------------
[figmess,t1,t2]=createfigmess;
set(t1,'string','Pre phase');
%-----------------------------------------------------------------------------------------

[analysis materials connec coor...           % reads input file
   dof displ TD]=read_input(pname,fname,t2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% THERMAL PROBLEM (AFTER TRANSIENT)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% creates info for thermal problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
E=1.;
nu=0.2;
k=1;                                         % conductivity
alpha=.2;                                    % dilat coeff
kappa=E/(3.*(1-nu));
A= E/((1+nu)*(1-2*nu))*[1-nu,nu,0;
                           nu,1-nu,0;
                           0,0,(1-2*nu)/2]; 


L=100;
e=3;
sinomega=0.423;

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Spring case: Dirichlet BC (Temperature)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tempD=20;                                    % imposed temperature

miny=min(coor(:,2));                         % min y coordinate 
maxy=max(coor(:,2));                         % max y coordinate
dofth=zeros(analysis.NN,1);                  % dof for thermal problem
temp=zeros(analysis.NN,1);
neqth=0;
for e=1:analysis.NE,
 nodes=connec(e,1:3);
 for n=1:3
  node=nodes(n);
  if coor(node,2)<miny+1.d-4                 % if node on lower surface
   dofth(node)=-node;
  elseif coor(node,2)>maxy-1.d-4,            % if node on upper surface
   dofth(node)=-node;
   temp(node)=tempD;
  else                                       % otherwise
   if dofth(node)==0                     
    neqth=neqth+1;
    dofth(node)=neqth;
   end
  end
 end 
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate of number of non-zero coefficients for thermal problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Ek=ones(analysis.NN,1);                      % initialisation of Ek
for e=1:analysis.NE,                         % loop over elements
  nodes=connec(e,1:3);
  Ek(nodes)=Ek(nodes)+1;                     % Ek is incremented on connectivity nodes
end
ncoeffs=sum(Ek);                             % sum of all the terms in Ek

C=spalloc(neqth,neqth,ncoeffs);              % allocates sparse matrix
F=zeros(neqth,1);                            % allocates rhs vector

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% assemblage phase: conductivity matrix and nodal forces due to imposed temp.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% Add conductivity matrix computation




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solution phase
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Th=C\F;                                      % solution of linear system
ic=find(dofth>0);                            % finds active dofs
temp(ic)=Th(dofth(ic));                      % and copies solution into temp

clear C F                                    % clears memory

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Post-processing phase: computation of nodal stresses via extrapolation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

thpost=0;                                    % if interested in seeing temp field
if thpost==1 
 stress=zeros(analysis.NN,3);               
 for e=1:analysis.NE,                        % computes temperatures at mid nodes as averag
   nodes=connec(e,:);
   Te=temp(nodes(1:3));
   displ(nodes(1:3),1)=Te;
   displ(nodes(4),1)=.5*(Te(1)+Te(2));
   displ(nodes(5),1)=.5*(Te(2)+Te(3));
   displ(nodes(6),1)=.5*(Te(3)+Te(1));       % and plots temperatues as first component of displ
 end   
%-----------------------------------------------------------------------------------------
 % graphical postprocessing
 close(figmess)
 interface('Initialize',[' mesh-undef| mesh-def| mesh-both| u1|'...
          ' u2| s11| s22| s12'],'post(infopost)'); 
%-----------------------------------------------------------------------------------------
% return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% THERMOELASTIC PROBLEM (AFTER TRANSIENT)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate of number of non-zero coefficients for T6
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Ek=3*ones(analysis.NN,1);                    % initialisation of Ek
for e=1:analysis.NE,                         % loop over elements
  nodes=connec(e,:);
  Ek(nodes)=Ek(nodes)+3;                     % Ek is incremented on connectivity nodes
end
ncoeffs=4*sum(Ek);                           % sum of all the terms in Ek
K=spalloc(analysis.neq,analysis.neq,ncoeffs);% allocates sparse matrix
F=zeros(analysis.neq,1);                     % allocates rhs vector

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% assemblage phase: system matrix and nodal forces due to imposed displ.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
D=2;                                         % problem dimensionality
Dne=D*analysis.ne;                           % number of nodal values in one surface el.
Dns=D*analysis.ns;                           % number of nodal values in one line el.
for e=1:analysis.NE,                         % stiffness matrix assemblage
  nodes=connec(e,:);                         % element nodes
  T=coor(nodes,:);                           % creates element
  Ke=stiff_linel_T6(T,A);
  dofe=reshape(dof(nodes,:)',[1,Dne]);       % list of dof associated to element
  pe=find(dofe>0);                           % gets position of unknown displ. compon.
  Ie=dofe(pe);                               % gets value of associated DOFs 
  K(Ie,Ie)=K(Ie,Ie)+Ke(pe,pe);               % matrix assemblage
end

%
% Add computation of right hand side (temperature)
% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solution phase
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

U=K\F;                                       % solution of linear system

clear K F                                    % clears memory

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Post-processing phase: computation of nodal stresses via extrapolation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ic=find(dof>0);                              % finds active dofs
displ(ic)=U(dof(ic));                        % and copies solution into displ
stress=zeros(analysis.NN,3);                 % initializes "stress"
counter=zeros(analysis.NN,1);                % initializes "counter"
for e=1:analysis.NE,                         % computes nodal stresses
  nodes=connec(e,:);
  T=coor(nodes,:);                           % creates element
  Ve=reshape(displ(nodes,:)',[Dne,1]);
% %%%%%%%%%%%%%%%%%%%


  stressG = zeros(3,3);                      % at gauss points

  
  
  % %%%%%%%%%%%%%%%%%%%
  stressN=G2N_T6(T,stressG);   
  stress(nodes,:)=stress(nodes,:)+stressN;   % adds stresses to triangle nodes
  counter(nodes)=counter(nodes)+1;
end   
for icomp=1:3
  stress(:,icomp)=stress(:,icomp)./counter;  % naive average of stresses
end

%-----------------------------------------------------------------------------------------
% graphical postprocessing
close(figmess)
interface('Initialize',[' mesh-undef| mesh-def| mesh-both| u1|'...
          ' u2| s11| s22| s12'],'post(infopost)'); 
%-----------------------------------------------------------------------------------------
